import argparse
import sys
from .utils import *
import typing


class Arguments:
    def __init__(self, tmp_args: argparse):
        benchmark = get_pure_name(tmp_args.benchmark)
        approximate_benchmark = get_pure_name(tmp_args.approximate_benchmark)
        self.__benchmark_name: str = benchmark
        self.__approximate_benchmark: str = approximate_benchmark
        self.__mc_samples: int = tmp_args.samples
        self.__metric: str = tmp_args.metric
        self.__precision: int = tmp_args.precision
        self.__strategy: str = tmp_args.strategy
        self.__optimization: str = tmp_args.optimization
        self.__experiment: str = tmp_args.experiment
        self.__pruning_percentage: int = tmp_args.pruning_percentage
        self.__clean = tmp_args.clean

    @property
    def benchmark_name(self):
        return self.__benchmark_name

    @property
    def approximate_benchmark(self):
        return self.__approximate_benchmark

    @property
    def num_samples(self):
        return self.__mc_samples

    @property
    def metric(self):
        return self.__metric

    @property
    def precision(self):
        return self.__precision

    @property
    def strategy(self):
        return self.__strategy

    @property
    def optimization(self):
        return self.__optimization

    @property
    def experiment(self):
        return self.__experiment

    @property
    def pruning_percentage(self):
        return self.__pruning_percentage

    # @property
    # def approach(self):
    #     return self.__approach

    @property
    def clean(self):
        return self.__clean

    def __repr__(self):
        return f'An object of class Arguments\n' \
               f'{self.benchmark_name = }\n' \
               f'{self.approximate_benchmark = }\n' \
               f'{self.num_samples = }\n' \
               f'{self.metric = }\n' \
               f'{self.precision = }\n' \
               f'{self.strategy = }\n' \
               f'{self.optimization = }\n' \
               f'{self.experiment = }\n' \
               f'{self.pruning_percentage = }\n' \
               f'{self.clean = }\n'

    @classmethod
    def parse(cls):
        my_parser = argparse.ArgumentParser(description='converts different formats to one another',
                                            prog=sys.argv[0],
                                            usage='%(prog)s benchmark-name|benchmark-path')

        my_parser.add_argument('benchmark',
                               type=str,
                               default=None,
                               help='benchmark-name')

        my_parser.add_argument('--samples', '-s',
                               default=100,
                               type=int,
                               help='number-of-monte-carlo-samples')
        my_parser.add_argument('--approximate_benchmark', '-app',
                               type=str,
                               default=None,
                               help='approximate-benchmark-name in gv/verilog format')
        my_parser.add_argument('--metric', '-metric',
                               type=str,
                               default=WAE,
                               help='the-desired-worst-case-error-metric')
        my_parser.add_argument('--precision', '-p',
                               type=int,
                               default=2,
                               help='number-of-decimal-points-for-wre')
        my_parser.add_argument('--strategy', '-strategy',
                               type=str,
                               default=MONOTONIC,
                               help='the-solver-strategy-to-find-metric')
        my_parser.add_argument('--optimization', '-opt',
                               type=str,
                               default=None,
                               help='the-solver-optimization (Solver, Optimize, Maximize)')
        my_parser.add_argument('--experiment', '-e',
                               type=str,
                               default=SINGLE,
                               help="the-experiment-name [SINGLE|QOR|RANDOM]")
        my_parser.add_argument('--pruning_percentage', '-pp',
                               type=int,
                               default=10,
                               help='gate-percentage-carved-out')

        my_parser.add_argument('--clean',
                               type=bool,
                               default=False)

        tmp_args = my_parser.parse_args()

        return Arguments(tmp_args)



from setuptools import setup, find_packages
import os


VERSION = '1.2.5'
DESCRIPTION = """
Paper Title: ErrorEval: an Open-Source Worst-Case-Error Evaluation Framework for Approximate Computing

Short Description: The open-source toolchain that proposes a methodology called ErrorEval which relies on 
SMT (Satisfiability Modulo Theories) solvers.

Authors: 
Morteza Rezaalipour, Università della Svizzera italiana (USI), Lugano, Switzerland
Lorenzo Ferretti, Micron Technology, San Jose, USA
Ilaria Scarabottolo, Università della Svizzera italiana (USI), Lugano, Switzerland
George A. Constantinides, Imperial College London, London, UK
and Laura Pozzi, Università della Svizzera italiana (USI), Lugano, Switzerland

Event name: (Computing Frontiers - Workshop on Open-Source Hardware) CF23-OSHW 23, May 9-11, 2023, Bologna, Italy
DOI: https://doi.org/10.1145/3587135.3591438
"""
INSTALL_REQUIREMENTS_FILE = 'install_requiremnets.txt'
INSTALL_REQUIREMENTS = []
with open(INSTALL_REQUIREMENTS_FILE, 'r') as IR:
    lines = IR.readlines()
    for line in lines:
        INSTALL_REQUIREMENTS.append(line.replace('\n', ""))


# Setting up
setup(
    name="z3log",
    version=VERSION,
    author="Morteza Rezaalipour (MorellRAP)",
    author_email="<rezaalipour.usi@gmail.com>",
    description=DESCRIPTION,
    packages=find_packages(),
    install_requires=INSTALL_REQUIREMENTS,
    keywords=['python', 'verilog', 'circuits', 'synthesis'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
) 



from .config.path import *
from .config.config import *


class Specs():
    def __init__(self, benchmark=None, approx_benchmark=None, experiment=None, strategy=MONOTONIC, metric=WAE,
                 num_samples=None, precision=2, optimization=None):
        self.__benchmark: str = benchmark
        self.__approximate_benchmark: str = approx_benchmark

        self.__experiment: str = experiment
        self.__strategy: str = strategy
        self.__optimization: str = optimization

        self.__metric: str = metric
        self.__precision: int = precision
        self.__num_samples: str = num_samples

    @property
    def benchmark(self):
        return self.__benchmark

    @property
    def approximate_benchmark(self):
        return self.__approximate_benchmark

    @property
    def experiment(self):
        return self.__experiment

    @property
    def strategy(self):
        return self.__strategy

    @property
    def optimization(self):
        return self.__optimization

    @property
    def metric(self):
        return self.__metric

    @property
    def precision(self):
        return self.__precision

    @property
    def num_samples(self):
        return self.__num_samples

    def __repr__(self):
        return f'An object of class Specs:\n' \
               f'{self.benchmark = }\n' \
               f'{self.approximate_benchmark = }\n' \
               f'{self.experiment = }\n' \
               f'{self.strategy = }\n' \
               f'{self.optimization = }\n' \
               f'{self.metric = }\n' \
               f'{self.precision = }\n' \
               f'{self.num_samples = }' \
               f''
